<?php

declare(strict_types = 1);

namespace Drupal\entity_share_bypass_fields\Plugin\EntityShareClient\Processor;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\entity_share_client\ImportProcessor\ImportProcessorPluginBase;
use Drupal\entity_share_client\RuntimeImportContext;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * General default data processor.
 *
 * @ImportProcessor(
 *   id = "entity_share_bypass_fields",
 *   label = @Translation("Bypass fields"),
 *   description = @Translation("Allow you to bypass missing fields and specific fields while synchronized entities.    "),
 *   stages = {
 *     "prepare_entity_data" = -100,
 *   },
 *   locked = false,
 * )
 */
class BypassFieldsProcessor extends ImportProcessorPluginBase implements PluginFormInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->logger = $container->get('logger.channel.entity_share_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'manual_bypassed_fields' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['manual_bypassed_fields'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Bypass fields manually'),
      '#description' => $this->t('Allow you to bypass some specific fields manually. Format : fields must be comma separated (example: "field_one,field_two,field_three")'),
      '#default_value' => $this->configuration['manual_bypassed_fields'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareEntityData(RuntimeImportContext $runtime_import_context, array &$entity_json_data) {
    // Get as array the list of manually bypassed field.
    $manuallyExcludedFields = explode(',', $this->configuration['manual_bypassed_fields']);

    // Entity Share response contains entity type and entity bundle under the
    // ID 'type' ($entity_json_data['type']) with the following format :
    // [entity_type]--[entity_bundle].
    $entityType = explode('--', $entity_json_data['type']);
    $entityUuid = $entity_json_data['id'];

    try {
      $entities = $this->entityTypeManager->getStorage($entityType[0])->loadByProperties(['uuid' => $entityUuid]);
      if (count($entities) > 1) {
        $this->logger->error('BypassFieldProcessor : There is more than 1 entity while loading uuid ' . $entityUuid . '.');
        return;
      }
      
      // We're loading by uuid so we must have only one entity returned by the
      // loadByProperties()
      $entity = reset($entities);
      if (empty($entity)) {
        $this->logger->error('BypassFieldProcessor : No entity found with uuid ' . $entityUuid . '.');
        return;
      }

      if (!($entity instanceof FieldableEntityInterface)) {
        $this->logger->error('BypassFieldProcessor : Entity found with uuid ' . $entityUuid . '. But it\'s not a FieldableEntityInterface instance.');
        return;
      }

      $entityAttributes = $entity_json_data['attributes'];
      foreach ($entityAttributes as $attributeId => $attribute) {
        if (in_array($attributeId, $manuallyExcludedFields)) {
          unset($entity_json_data['attributes'][$attributeId]);
          continue;
        }

        if (str_starts_with($attributeId, 'field_') && !($entity->hasField($attributeId))) {
          unset($entity_json_data['attributes'][$attributeId]);
        }
      }
    } catch (\Exception $e) {
      $this->logger->critical('BypassFieldProcessor failed with error: ' . $e->getMessage());
    }
  }

}
